@@ -57,6 +57,10 @@
 #include <qwindow.h>
 #include <private/qhighdpiscaling_p.h>
 
+#ifdef Q_OS_WIN
+#include <QtCore/qt_windows.h>
+#endif
+
 using namespace Qt;
 
 QT_BEGIN_NAMESPACE
@@ -73,7 +77,7 @@ QVideoWidgetControlBackend::QVideoWidgetControlBackend(
     connect(control, SIGNAL(fullScreenChanged(bool)), widget, SLOT(_q_fullScreenChanged(bool)));
 
     QBoxLayout *layout = new QVBoxLayout;
-    layout->setMargin(0);
+    layout->setContentsMargins(0, 0, 0, 0);
     layout->setSpacing(0);
 
     QWidget *videoWidget = control->videoWidget();
@@ -324,6 +328,7 @@ QWindowVideoWidgetBackend::QWindowVideoWidgetBackend(
 
     control->setWinId(widget->winId());
 #if defined(Q_OS_WIN)
+    // Disable updates to avoid flickering while resizing/moving.
     m_widget->setUpdatesEnabled(false);
 #endif
 }
@@ -622,7 +627,7 @@ QVideoWidget::QVideoWidget(QVideoWidgetPrivate &dd, QWidget *parent)
     d_ptr->q_ptr = this;
 
     QPalette palette = QWidget::palette();
-    palette.setColor(QPalette::Background, Qt::black);
+    palette.setColor(QPalette::Window, Qt::black);
     setPalette(palette);
 }
 
@@ -1001,7 +1006,28 @@ void QVideoWidget::paintEvent(QPaintEvent *event)
     }
 }
 
-#include "moc_qvideowidget.cpp"
-#include "moc_qvideowidget_p.cpp"
+#if defined(Q_OS_WIN)
+#  if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+bool QVideoWidget::nativeEvent(const QByteArray &eventType, void *message, qintptr *result)
+#  else
+bool QVideoWidget::nativeEvent(const QByteArray &eventType, void *message, long *result)
+#  endif
+{
+    Q_D(QVideoWidget);
+    Q_UNUSED(eventType);
+    Q_UNUSED(result);
+
+    MSG *mes = reinterpret_cast<MSG *>(message);
+    if (mes->message == WM_PAINT || mes->message == WM_ERASEBKGND) {
+        if (d->windowBackend)
+            d->windowBackend->showEvent();
+    }
+
+    return false;
+}
+#endif
+
 QT_END_NAMESPACE
 
+#include "moc_qvideowidget.cpp"
+#include "moc_qvideowidget_p.cpp"
